@@ -25,6 +25,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
+#include <X11/Xatom.h>
 
 #include "canberra.h"
 #include "canberra-gtk.h"
@@ -167,6 +168,34 @@ static GtkWindow* get_toplevel(GtkWidget *w) {
     return GTK_WINDOW(w);
 }
 
+static gint window_get_desktop(GdkDisplay *d, GdkWindow *w) {
+    Atom type_return;
+    gint format_return;
+    gulong nitems_return;
+    gulong bytes_after_return;
+    guchar *data = NULL;
+    gint ret = -1;
+
+    if (XGetWindowProperty(GDK_DISPLAY_XDISPLAY(d), GDK_WINDOW_XID(w),
+                           gdk_x11_get_xatom_by_name_for_display(d, "_NET_WM_DESKTOP"),
+                           0, G_MAXLONG, False, XA_CARDINAL, &type_return,
+                           &format_return, &nitems_return, &bytes_after_return,
+                           &data) != Success)
+        return -1;
+
+    if (type_return == XA_CARDINAL && format_return == 32 && data) {
+        guint32 desktop = *(guint32*) data;
+
+        if (desktop != 0xFFFFFFFF)
+            ret = (gint) desktop;
+    }
+
+    if (type_return != None && data != NULL)
+        XFree(data);
+
+    return ret;
+}
+
 /**
  * ca_gtk_proplist_set_for_widget:
  * @p: The proplist to store these sound event properties in
@@ -183,9 +212,6 @@ int ca_gtk_proplist_set_for_widget(ca_proplist *p, GtkWidget *widget) {
     GtkWindow *w;
     int ret;
     const char *t, *role;
-    GdkWindow *dw;
-    GdkScreen *screen;
-    gint x = 0, y = 0, width = 0, height = 0, screen_width = 0, screen_height = 0;
 
     ca_return_val_if_fail(p, CA_ERROR_INVALID);
     ca_return_val_if_fail(widget, CA_ERROR_INVALID);
@@ -217,66 +243,85 @@ int ca_gtk_proplist_set_for_widget(ca_proplist *p, GtkWidget *widget) {
         if ((ret = ca_proplist_sets(p, CA_PROP_WINDOW_ICON_NAME, t)) < 0)
             return ret;
 
-    if ((dw = GTK_WIDGET(w)->window))
-        if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_X11_XID, "%lu", (unsigned long) GDK_WINDOW_XID(dw))) < 0)
-            return ret;
+    if (GTK_WIDGET_REALIZED(GTK_WIDGET(w))) {
+        GdkWindow *dw = NULL;
+        GdkScreen *screen = NULL;
+        GdkDisplay *display = NULL;
+        gint x = -1, y = -1, width = -1, height = -1, screen_width = -1, screen_height = -1;
 
-    if ((screen = gtk_widget_get_screen(widget))) {
-
-        if ((t = gdk_display_get_name(gdk_screen_get_display(screen))))
-            if ((ret = ca_proplist_sets(p, CA_PROP_WINDOW_X11_DISPLAY, t)) < 0)
+        if ((dw = gtk_widget_get_window(GTK_WIDGET(w))))
+            if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_X11_XID, "%lu", (unsigned long) GDK_WINDOW_XID(dw))) < 0)
                 return ret;
 
-        if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_X11_SCREEN, "%i", gdk_screen_get_number(screen))) < 0)
-            return ret;
+        if ((display = gtk_widget_get_display(GTK_WIDGET(w)))) {
+            if ((t = gdk_display_get_name(display)))
+                if ((ret = ca_proplist_sets(p, CA_PROP_WINDOW_X11_DISPLAY, t)) < 0)
+                    return ret;
 
-        if (dw)
-            if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_X11_MONITOR, "%i", gdk_screen_get_monitor_at_window(screen, dw))) < 0)
+            if (dw)  {
+                gint desktop = window_get_desktop(display, dw);
+
+                if (desktop >= 0)
+                    if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_DESKTOP, "%i", desktop)) < 0)
+                        return ret;
+            }
+        }
+
+        if ((screen = gtk_widget_get_screen(GTK_WIDGET(w)))) {
+
+            if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_X11_SCREEN, "%i", gdk_screen_get_number(screen))) < 0)
                 return ret;
-    }
 
-    /* FIXME, this might cause a round trip */
+            if (dw)
+                if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_X11_MONITOR, "%i", gdk_screen_get_monitor_at_window(screen, dw))) < 0)
+                    return ret;
+        }
 
-    if (dw) {
-        gdk_window_get_origin(dw, &x, &y);
+        /* FIXME, this might cause a round trip */
 
-        if (x >= 0)
-            if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_X, "%i", x)) < 0)
+        if (dw) {
+            gdk_window_get_origin(dw, &x, &y);
+
+            if (x >= 0)
+                if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_X, "%i", x)) < 0)
+                    return ret;
+            if (y >= 0)
+                if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_Y, "%i", y)) < 0)
+                    return ret;
+        }
+
+        gtk_window_get_size(w, &width, &height);
+
+        if (width > 0)
+            if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_WIDTH, "%i", width)) < 0)
                 return ret;
-        if (y >= 0)
-            if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_Y, "%i", y)) < 0)
+        if (height > 0)
+            if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_HEIGHT, "%i", height)) < 0)
                 return ret;
-    }
 
-    gtk_window_get_size(w, &width, &height);
+        if (x >= 0 && width > 0) {
+            screen_width = gdk_screen_get_width(gtk_widget_get_screen(GTK_WIDGET(w)));
 
-    if (width > 0)
-        if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_WIDTH, "%i", width)) < 0)
-            return ret;
-    if (height > 0)
-        if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_HEIGHT, "%i", height)) < 0)
-            return ret;
+            x += width/2;
+            x = CA_CLAMP(x, 0, screen_width-1);
 
-    if (x > 0 && width > 0) {
-        x += width/2;
+            /* We use these strange format strings here to avoid that libc
+             * applies locale information on the formatting of floating
+             * numbers. */
 
-        screen_width = gdk_screen_get_width(gtk_widget_get_screen(GTK_WIDGET(w)));
+            if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_HPOS, "%i.%03i", (int) (x/(screen_width-1)), (int) (1000.0*x/(screen_width-1)) % 1000)) < 0)
+                return ret;
+        }
 
-        /* We use these strange format strings here to avoid that libc
-         * applies locale information on the formatting of floating
-         * numbers. */
+        if (y >= 0 && height > 0) {
+            screen_height = gdk_screen_get_height(gtk_widget_get_screen(GTK_WIDGET(w)));
 
-        if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_HPOS, "%i.%03i", (int) (x/screen_width), (int) (1000.0*x/screen_width) % 1000)) < 0)
-            return ret;
-    }
+            y += height/2;
+            y = CA_CLAMP(y, 0, screen_height-1);
 
-    if (y > 0 && height > 0) {
-        y += height/2;
-
-        screen_height = gdk_screen_get_height(gtk_widget_get_screen(GTK_WIDGET(w)));
-
-        if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_VPOS, "%i.%03i", (int) (y/screen_height), (int) (1000.0*y/screen_height) % 1000)) < 0)
-            return ret;
+            if ((ret = ca_proplist_setf(p, CA_PROP_WINDOW_VPOS, "%i.%03i", (int) (y/(screen_height-1)), (int) (1000.0*y/(screen_height-1)) % 1000)) < 0)
+                return ret;
+        }
     }
 
     return CA_SUCCESS;
